@@ -19,14 +19,21 @@ mod rust_name_tracker;
 
 use crate::{
     conversion::{
-        convert_error::ConvertErrorWithContext, convert_error::ErrorContext,
-        error_reporter::add_api_or_report_error, parse::type_converter::TypeConversionContext,
+        analysis::{
+            has_attr,
+            type_converter::{add_analysis, TypeConversionContext, TypeConverter},
+        },
+        api::{ApiName, FuncToConvert},
+        convert_error::ConvertErrorWithContext,
+        convert_error::ErrorContext,
+        error_reporter::convert_apis,
     },
     known_types::known_types,
+    types::validate_ident_ok_for_rust,
 };
 use std::collections::{HashMap, HashSet};
 
-use autocxx_parser::{TypeConfig, UnsafePolicy};
+use autocxx_parser::{IncludeCppConfig, UnsafePolicy};
 use function_wrapper::{FunctionWrapper, FunctionWrapperPayload, TypeConversionPolicy};
 use proc_macro2::Span;
 use syn::{
@@ -36,9 +43,8 @@ use syn::{
 
 use crate::{
     conversion::{
-        api::{Api, ApiAnalysis, ApiDetail, FuncToConvert, TypeKind, UnanalyzedApi},
+        api::{AnalysisPhase, Api, TypeKind, UnanalyzedApi},
         codegen_cpp::AdditionalNeed,
-        parse::type_converter::TypeConverter,
         ConvertError,
     },
     types::{make_ident, validate_ident_ok_for_cxx, Namespace, QualifiedName},
@@ -49,7 +55,10 @@ use self::{
     rust_name_tracker::RustNameTracker,
 };
 
-use super::pod::PodAnalysis;
+use super::{
+    pod::{PodAnalysis, PodStructAnalysisBody},
+    tdef::TypedefAnalysisBody,
+};
 
 pub(crate) enum MethodKind {
     Normal,
@@ -84,10 +93,10 @@ pub(crate) struct FnAnalysisBody {
     pub(crate) kind: FnKind,
     pub(crate) ret_type: ReturnType,
     pub(crate) param_details: Vec<ArgumentAnalysis>,
-    pub(crate) cpp_call_name: String,
     pub(crate) requires_unsafe: bool,
     pub(crate) vis: Visibility,
     pub(crate) cpp_wrapper: Option<AdditionalNeed>,
+    pub(crate) deps: HashSet<QualifiedName>,
 }
 
 pub(crate) struct ArgumentAnalysis {
@@ -109,8 +118,9 @@ struct ReturnTypeAnalysis {
 
 pub(crate) struct FnAnalysis;
 
-impl ApiAnalysis for FnAnalysis {
-    type TypeAnalysis = TypeKind;
+impl AnalysisPhase for FnAnalysis {
+    type TypedefAnalysis = TypedefAnalysisBody;
+    type StructAnalysis = PodStructAnalysisBody;
     type FunAnalysis = FnAnalysisBody;
 }
 
@@ -118,65 +128,61 @@ pub(crate) struct FnAnalyzer<'a> {
     unsafe_policy: UnsafePolicy,
     rust_name_tracker: RustNameTracker,
     extra_apis: Vec<UnanalyzedApi>,
-    type_converter: &'a mut TypeConverter<'a>,
+    type_converter: TypeConverter<'a>,
     bridge_name_tracker: BridgeNameTracker,
     pod_safe_types: HashSet<QualifiedName>,
-    type_config: &'a TypeConfig,
-    incomplete_types: HashSet<QualifiedName>,
+    config: &'a IncludeCppConfig,
     overload_trackers_by_mod: HashMap<Namespace, OverloadTracker>,
     generate_utilities: bool,
 }
 
-struct FnAnalysisResult(FnAnalysisBody, Ident, HashSet<QualifiedName>);
-
 impl<'a> FnAnalyzer<'a> {
     pub(crate) fn analyze_functions(
         apis: Vec<Api<PodAnalysis>>,
         unsafe_policy: UnsafePolicy,
-        type_converter: &'a mut TypeConverter<'a>,
-        type_database: &'a TypeConfig,
+        config: &'a IncludeCppConfig,
     ) -> Vec<Api<FnAnalysis>> {
         let mut me = Self {
             unsafe_policy,
             rust_name_tracker: RustNameTracker::new(),
             extra_apis: Vec::new(),
-            type_converter,
+            type_converter: TypeConverter::new(config, &apis),
             bridge_name_tracker: BridgeNameTracker::new(),
-            type_config: type_database,
-            incomplete_types: Self::build_incomplete_type_set(&apis),
+            config,
             overload_trackers_by_mod: HashMap::new(),
             pod_safe_types: Self::build_pod_safe_type_set(&apis),
             generate_utilities: Self::should_generate_utilities(&apis),
         };
         let mut results = Vec::new();
-        for api in apis {
-            add_api_or_report_error(api.typename(), &mut results, || me.analyze_fn_api(api));
-        }
-        results.extend(me.extra_apis.into_iter().map(Self::make_extra_api_nonpod));
+        convert_apis(
+            apis,
+            &mut results,
+            |name, fun, _| me.analyze_foreign_fn(name, fun),
+            Api::struct_unchanged,
+            Api::enum_unchanged,
+            Api::typedef_unchanged,
+        );
+        results.extend(me.extra_apis.into_iter().map(add_analysis));
         results
     }
 
     fn should_generate_utilities(apis: &[Api<PodAnalysis>]) -> bool {
         apis.iter()
-            .any(|api| matches!(api.detail, ApiDetail::StringConstructor))
-    }
-
-    fn build_incomplete_type_set(apis: &[Api<PodAnalysis>]) -> HashSet<QualifiedName> {
-        apis.iter()
-            .filter_map(|api| match api.detail {
-                ApiDetail::ForwardDeclaration => Some(api.typename()),
-                _ => None,
-            })
-            .collect()
+            .any(|api| matches!(api, Api::StringConstructor { .. }))
     }
 
     fn build_pod_safe_type_set(apis: &[Api<PodAnalysis>]) -> HashSet<QualifiedName> {
         apis.iter()
-            .filter_map(|api| match api.detail {
-                ApiDetail::Type {
-                    bindgen_mod_item: _,
-                    analysis: TypeKind::Pod,
-                } => Some(api.typename()),
+            .filter_map(|api| match api {
+                Api::Struct {
+                    analysis:
+                        PodStructAnalysisBody {
+                            kind: TypeKind::Pod,
+                            ..
+                        },
+                    ..
+                } => Some(api.name().clone()),
+                Api::Enum { .. } => Some(api.name().clone()),
                 _ => None,
             })
             .chain(
@@ -195,72 +201,6 @@ impl<'a> FnAnalyzer<'a> {
             .collect()
     }
 
-    /// Processing functions sometimes results in new types being materialized.
-    /// In future, if we wanted to make these POD, we'd probably want to create
-    /// a new analysis phase prior to the POD analysis which materializes these types.
-    fn make_extra_api_nonpod(api: UnanalyzedApi) -> Api<FnAnalysis> {
-        let new_detail = match api.detail {
-            ApiDetail::ConcreteType { rs_definition } => ApiDetail::ConcreteType { rs_definition },
-            _ => panic!("Function analysis created an extra API which wasn't a concrete type"),
-        };
-        Api {
-            name: api.name,
-            // An "extra API" is unlikely to have an `original_name, but we
-            // don't need to knoww that here -- we can simply pass on
-            // `api.original_name`.
-            original_name: api.original_name,
-            deps: api.deps,
-            detail: new_detail,
-        }
-    }
-
-    fn analyze_fn_api(
-        &mut self,
-        api: Api<PodAnalysis>,
-    ) -> Result<Option<Api<FnAnalysis>>, ConvertErrorWithContext> {
-        let mut new_deps = api.deps.clone();
-        let mut new_id = api.name.get_final_ident();
-        let api_detail = match api.detail {
-            // No changes to any of these...
-            ApiDetail::ConcreteType { rs_definition } => ApiDetail::ConcreteType { rs_definition },
-            ApiDetail::StringConstructor => ApiDetail::StringConstructor,
-            ApiDetail::Function { fun, analysis: _ } => {
-                let analysis = self.analyze_foreign_fn(
-                    &api.name.get_namespace(),
-                    &fun,
-                    api.original_name.clone(),
-                )?;
-                match analysis {
-                    None => return Ok(None),
-                    Some(FnAnalysisResult(analysis, id, fn_deps)) => {
-                        new_deps = fn_deps;
-                        new_id = id;
-                        ApiDetail::Function { fun, analysis }
-                    }
-                }
-            }
-            ApiDetail::Const { const_item } => ApiDetail::Const { const_item },
-            ApiDetail::Typedef { payload } => ApiDetail::Typedef { payload },
-            ApiDetail::CType { typename } => ApiDetail::CType { typename },
-            // Just changes to this one...
-            ApiDetail::Type {
-                bindgen_mod_item,
-                analysis,
-            } => ApiDetail::Type {
-                bindgen_mod_item,
-                analysis,
-            },
-            ApiDetail::ForwardDeclaration => ApiDetail::ForwardDeclaration,
-            ApiDetail::IgnoredItem { err, ctx } => ApiDetail::IgnoredItem { err, ctx },
-        };
-        Ok(Some(Api {
-            name: QualifiedName::new(api.name.get_namespace(), new_id),
-            original_name: api.original_name,
-            deps: new_deps,
-            detail: api_detail,
-        }))
-    }
-
     fn convert_boxed_type(
         &mut self,
         ty: Box<Type>,
@@ -272,7 +212,6 @@ impl<'a> FnAnalyzer<'a> {
             ns,
             &TypeConversionContext::CxxOuterType {
                 convert_ptrs_to_references,
-                forward_declarations: &self.incomplete_types,
             },
         )?;
         self.extra_apis.extend(annotated.extra_apis);
@@ -298,7 +237,7 @@ impl<'a> FnAnalyzer<'a> {
     }
 
     fn is_on_allowlist(&self, type_name: &QualifiedName) -> bool {
-        self.type_config.is_on_allowlist(&type_name.to_cpp_name())
+        self.config.is_on_allowlist(&type_name.to_cpp_name())
     }
 
     fn should_be_unsafe(&self) -> bool {
@@ -313,7 +252,7 @@ impl<'a> FnAnalyzer<'a> {
     /// We aim for the simplest case but, for example:
     /// * We'll need a C++ wrapper for static methods
     /// * We'll need a C++ wrapper for parameters which need to be wrapped and unwrapped
-    ///   to [UniquePtr]
+    ///   to [cxx::UniquePtr]
     /// * We'll need a Rust wrapper if we've got a C++ wrapper and it's a method.
     /// * We may need wrappers if names conflict.
     /// etc.
@@ -322,12 +261,13 @@ impl<'a> FnAnalyzer<'a> {
     /// The output of this analysis phase is used by both Rust and C++ codegen.
     fn analyze_foreign_fn(
         &mut self,
-        ns: &Namespace,
-        func_information: &FuncToConvert,
-        original_name: Option<String>,
-    ) -> Result<Option<FnAnalysisResult>, ConvertErrorWithContext> {
+        name: ApiName,
+        func_information: Box<FuncToConvert>,
+    ) -> Result<Option<Api<FnAnalysis>>, ConvertErrorWithContext> {
         let fun = &func_information.item;
         let virtual_this = &func_information.virtual_this_type;
+        let mut cpp_name = name.cpp_name.clone();
+        let ns = name.name.get_namespace();
 
         // Let's gather some pre-wisdom about the name of the function.
         // We're shortly going to plunge into analyzing the parameters,
@@ -337,7 +277,7 @@ impl<'a> FnAnalyzer<'a> {
         if initial_rust_name.ends_with("_destructor") {
             return Ok(None);
         }
-        let diagnostic_display_name = original_name.as_ref().unwrap_or(&initial_rust_name);
+        let diagnostic_display_name = cpp_name.as_ref().unwrap_or(&initial_rust_name);
 
         // Now let's analyze all the parameters.
         // See if any have annotations which our fork of bindgen has craftily inserted...
@@ -375,19 +315,29 @@ impl<'a> FnAnalyzer<'a> {
 
         // End of parameter processing.
         // Work out naming, part one.
-        let mut rust_name;
+        // The Rust name... it's more complicated.
         // bindgen may have mangled the name either because it's invalid Rust
         // syntax (e.g. a keyword like 'async') or it's an overload.
         // If the former, we respect that mangling. If the latter, we don't,
         // because we'll add our own overload counting mangling later.
-        let name_probably_invalid_in_rust =
-            original_name.is_some() && initial_rust_name.ends_with('_');
-        // The C++ call name will always be whatever bindgen tells us.
-        let cpp_call_name = original_name.unwrap_or_else(|| initial_rust_name.clone());
-        let ideal_rust_name = if name_probably_invalid_in_rust {
-            initial_rust_name
-        } else {
-            cpp_call_name.clone()
+        // Cases:
+        //   function, IRN=foo,    CN=<none>                    output: foo    case 1
+        //   function, IRN=move_,  CN=move   (keyword problem)  output: move_  case 2
+        //   function, IRN=foo1,   CN=foo    (overload)         output: foo    case 3
+        //   method,   IRN=A_foo,  CN=foo                       output: foo    case 4
+        //   method,   IRN=A_move, CN=move   (keyword problem)  output: move_  case 5
+        //   method,   IRN=A_foo1, CN=foo    (overload)         output: foo    case 6
+        let ideal_rust_name = match &cpp_name {
+            None => initial_rust_name, // case 1
+            Some(cpp_name) => {
+                if initial_rust_name.ends_with('_') {
+                    initial_rust_name // case 2
+                } else if validate_ident_ok_for_rust(&cpp_name).is_err() {
+                    format!("{}_", cpp_name) // case 5
+                } else {
+                    cpp_name.to_string() // cases 3, 4, 6
+                }
+            }
         };
 
         // Let's spend some time figuring out the kind of this function (i.e. method,
@@ -401,7 +351,7 @@ impl<'a> FnAnalyzer<'a> {
             (false, self_ty)
         };
 
-        let (kind, error_context) = if let Some(self_ty) = self_ty {
+        let (kind, error_context, rust_name) = if let Some(self_ty) = self_ty {
             // Some kind of method.
             if !self.is_on_allowlist(&self_ty) {
                 // Bindgen will output methods for types which have been encountered
@@ -410,6 +360,7 @@ impl<'a> FnAnalyzer<'a> {
                 // It may, for instance, be a private type.
                 return Ok(None);
             }
+
             // Method or static method.
             let type_ident = self_ty.get_final_item();
             // bindgen generates methods with the name:
@@ -419,7 +370,7 @@ impl<'a> FnAnalyzer<'a> {
             // We want to feed cxx methods with just the method name, so let's
             // strip off the class name.
             let overload_tracker = self.overload_trackers_by_mod.entry(ns.clone()).or_default();
-            rust_name = overload_tracker.get_method_real_name(&type_ident, ideal_rust_name);
+            let mut rust_name = overload_tracker.get_method_real_name(&type_ident, ideal_rust_name);
             let method_kind = if rust_name.starts_with(&type_ident) {
                 // It's a constructor. bindgen generates
                 // fn new(this: *Type, ...args)
@@ -438,7 +389,7 @@ impl<'a> FnAnalyzer<'a> {
             } else if is_static_method {
                 MethodKind::Static
             } else if param_details.iter().any(|pd| pd.is_virtual) {
-                if Self::has_attr(&fun, "bindgen_pure_virtual") {
+                if has_attr(&fun.attrs, "bindgen_pure_virtual") {
                     MethodKind::PureVirtual
                 } else {
                     MethodKind::Virtual
@@ -450,13 +401,21 @@ impl<'a> FnAnalyzer<'a> {
                 self_ty: self_ty.get_final_ident(),
                 method: make_ident(&rust_name),
             };
-            (FnKind::Method(self_ty, method_kind), error_context)
+            (
+                FnKind::Method(self_ty, method_kind),
+                error_context,
+                rust_name,
+            )
         } else {
             // Not a method.
             // What shall we call this function? It may be overloaded.
             let overload_tracker = self.overload_trackers_by_mod.entry(ns.clone()).or_default();
-            rust_name = overload_tracker.get_function_real_name(ideal_rust_name);
-            (FnKind::Function, ErrorContext::Item(make_ident(&rust_name)))
+            let rust_name = overload_tracker.get_function_real_name(ideal_rust_name);
+            (
+                FnKind::Function,
+                ErrorContext::Item(make_ident(&rust_name)),
+                rust_name,
+            )
         };
 
         // The name we use within the cxx::bridge mod may be different
@@ -470,13 +429,16 @@ impl<'a> FnAnalyzer<'a> {
             &rust_name,
             &ns,
         );
+        if cxxbridge_name != rust_name && cpp_name.is_none() {
+            cpp_name = Some(rust_name.clone());
+        }
         let mut cxxbridge_name = make_ident(&cxxbridge_name);
 
         // If we encounter errors from here on, we can give some context around
         // where the error occurred such that we can put a marker in the output
         // Rust code to indicate that a problem occurred (benefiting people using
         // rust-analyzer or similar). Make a closure to make this easy.
-        let contextualize_error = |err| ConvertErrorWithContext(err, Some(error_context));
+        let contextualize_error = |err| ConvertErrorWithContext(err, Some(error_context.clone()));
 
         // Now we can add context to the error, check for a couple of error
         // cases. First, see if any of the parameters are trouble.
@@ -487,7 +449,7 @@ impl<'a> FnAnalyzer<'a> {
             }
         }
         // Second, reject any functions handling types which we flake out on.
-        if Self::has_attr(&fun, "bindgen_unused_template_param_in_arg_or_return") {
+        if has_attr(&fun.attrs, "bindgen_unused_template_param_in_arg_or_return") {
             return Err(contextualize_error(ConvertError::UnusedTemplateParam));
         }
 
@@ -498,6 +460,16 @@ impl<'a> FnAnalyzer<'a> {
             ));
         }
 
+        match kind {
+            FnKind::Method(_, MethodKind::Static) => {}
+            FnKind::Method(ref self_ty, _) => {
+                if !known_types().is_cxx_acceptable_receiver(&self_ty) {
+                    return Err(contextualize_error(ConvertError::UnsupportedReceiver));
+                }
+            }
+            _ => {}
+        };
+
         // Analyze the return type, just as we previously did for the
         // parameters.
         let mut return_analysis = if let FnKind::Method(ref self_ty, MethodKind::Constructor) = kind
@@ -516,13 +488,8 @@ impl<'a> FnAnalyzer<'a> {
                 deps: these_deps,
             }
         } else {
-            // We can't easily use map_err below because the borrow checker can't
-            // prove we don't use contextualize_error more than once.
-            let r = self.convert_return_type(&fun.sig.output, &ns, reference_return);
-            match r {
-                Err(err) => return Err(contextualize_error(err)),
-                Ok(r) => r,
-            }
+            self.convert_return_type(&fun.sig.output, &ns, reference_return)
+                .map_err(contextualize_error)?
         };
         let mut deps = params_deps;
         deps.extend(return_analysis.deps.drain());
@@ -545,6 +512,10 @@ impl<'a> FnAnalyzer<'a> {
         let ret_type_conversion_needed = ret_type_conversion
             .as_ref()
             .map_or(false, |x| x.cpp_work_needed());
+        // See https://github.com/dtolnay/cxx/issues/878 for the reason for this next line.
+        let effective_cpp_name = cpp_name.as_ref().unwrap_or(&rust_name);
+        let cpp_name_incompatible_with_cxx =
+            validate_ident_ok_for_rust(&effective_cpp_name).is_err();
         // If possible, we'll put knowledge of the C++ API directly into the cxx::bridge
         // mod. However, there are various circumstances where cxx can't work with the existing
         // C++ API and we need to create a C++ wrapper function which is more cxx-compliant.
@@ -557,13 +528,14 @@ impl<'a> FnAnalyzer<'a> {
             FnKind::Method(..) if cxxbridge_name != rust_name => true,
             _ if param_conversion_needed => true,
             _ if ret_type_conversion_needed => true,
+            _ if cpp_name_incompatible_with_cxx => true,
             _ => false,
         };
 
         let cpp_wrapper = if wrapper_function_needed {
             // Generate a new layer of C++ code to wrap/unwrap parameters
             // and return values into/out of std::unique_ptrs.
-            let cpp_construction_ident = make_ident(&cpp_call_name);
+            let cpp_construction_ident = make_ident(&effective_cpp_name);
             let joiner = if cxxbridge_name.to_string().ends_with('_') {
                 ""
             } else {
@@ -653,8 +625,9 @@ impl<'a> FnAnalyzer<'a> {
             }
         };
 
-        Ok(Some(FnAnalysisResult(
-            FnAnalysisBody {
+        Ok(Some(Api::Function {
+            fun: func_information,
+            analysis: FnAnalysisBody {
                 cxxbridge_name,
                 rust_name,
                 rust_rename_strategy,
@@ -662,14 +635,16 @@ impl<'a> FnAnalyzer<'a> {
                 kind,
                 ret_type,
                 param_details,
-                cpp_call_name,
                 requires_unsafe,
                 vis,
                 cpp_wrapper,
+                deps,
             },
-            id,
-            deps,
-        )))
+            name: ApiName {
+                cpp_name,
+                name: QualifiedName::new(ns, id),
+            },
+        }))
     }
 
     fn convert_fn_arg(
@@ -853,22 +828,19 @@ impl<'a> FnAnalyzer<'a> {
         }
         (ref_params, ref_return)
     }
-
-    fn has_attr(fun: &ForeignItemFn, attr_name: &str) -> bool {
-        fun.attrs.iter().any(|at| at.path.is_ident(attr_name))
-    }
 }
 
 impl Api<FnAnalysis> {
     pub(crate) fn typename_for_allowlist(&self) -> QualifiedName {
-        match &self.detail {
-            ApiDetail::Function { fun: _, analysis } => match analysis.kind {
+        match &self {
+            Api::Function { analysis, .. } => match analysis.kind {
                 FnKind::Method(ref self_ty, _) => self_ty.clone(),
-                FnKind::Function => {
-                    QualifiedName::new(&self.name.get_namespace(), make_ident(&analysis.rust_name))
-                }
+                FnKind::Function => QualifiedName::new(
+                    &self.name().get_namespace(),
+                    make_ident(&analysis.rust_name),
+                ),
             },
-            _ => self.typename(),
+            _ => self.name().clone(),
         }
     }
 
@@ -879,17 +851,39 @@ impl Api<FnAnalysis> {
     /// more C++ is needed (so it can add #includes in the cxx mod).
     /// And we can't answer the question _prior_ to this function analysis phase.
     pub(crate) fn additional_cpp(&self) -> Option<AdditionalNeed> {
-        match &self.detail {
-            ApiDetail::Function { fun: _, analysis } => analysis.cpp_wrapper.clone(),
-            ApiDetail::StringConstructor => Some(AdditionalNeed::MakeStringConstructor),
-            ApiDetail::ConcreteType { rs_definition } => {
+        match &self {
+            Api::Function { analysis, .. } => analysis.cpp_wrapper.clone(),
+            Api::StringConstructor { .. } => Some(AdditionalNeed::MakeStringConstructor),
+            Api::ConcreteType { rs_definition, .. } => {
                 Some(AdditionalNeed::ConcreteTemplatedTypeTypedef(
-                    self.name.clone(),
+                    self.name().clone(),
                     rs_definition.clone(),
                 ))
             }
-            ApiDetail::CType { typename } => Some(AdditionalNeed::CTypeTypedef(typename.clone())),
+            Api::CType { typename, .. } => Some(AdditionalNeed::CTypeTypedef(typename.clone())),
             _ => None,
         }
     }
+
+    pub(crate) fn cxxbridge_name(&self) -> Option<Ident> {
+        match self {
+            Api::Function { ref analysis, .. } => Some(analysis.cxxbridge_name.clone()),
+            Api::StringConstructor { .. } | Api::Const { .. } | Api::IgnoredItem { .. } => None,
+            _ => Some(self.name().get_final_ident()),
+        }
+    }
+
+    /// Any dependencies on other APIs which this API has.
+    pub(crate) fn deps(&self) -> Box<dyn Iterator<Item = &QualifiedName> + '_> {
+        match self {
+            Api::Typedef {
+                old_tyname,
+                analysis: TypedefAnalysisBody { deps, .. },
+                ..
+            } => Box::new(old_tyname.iter().chain(deps.iter())),
+            Api::Struct { analysis, .. } => Box::new(analysis.field_deps.iter()),
+            Api::Function { analysis, .. } => Box::new(analysis.deps.iter()),
+            _ => Box::new(std::iter::empty()),
+        }
+    }
 }
