@@ -15,20 +15,21 @@
 use std::collections::HashSet;
 
 use super::fun::{FnAnalysis, FnAnalysisBody, FnKind};
+use crate::conversion::api::ApiName;
 use crate::conversion::{convert_error::ErrorContext, ConvertError};
-use crate::{
-    conversion::api::{Api, ApiDetail},
-    known_types,
-};
+use crate::{conversion::api::Api, known_types};
 
 /// Remove any APIs which depend on other items which have been ignored.
+/// We also eliminate any APIs that depend on some type that we just don't
+/// know about at all. In either case, we don't simply remove the type, but instead
+/// replace it with an error marker.
 pub(crate) fn filter_apis_by_ignored_dependents(
     mut apis: Vec<Api<FnAnalysis>>,
 ) -> Vec<Api<FnAnalysis>> {
     let (ignored_items, valid_items): (Vec<&Api<_>>, Vec<&Api<_>>) = apis.iter().partition(|api| {
         matches!(
-            api.detail,
-            ApiDetail::IgnoredItem {
+            api,
+            Api::IgnoredItem {
                 ctx: ErrorContext::Item(..),
                 ..
             }
@@ -36,26 +37,29 @@ pub(crate) fn filter_apis_by_ignored_dependents(
     });
     let mut ignored_items: HashSet<_> = ignored_items
         .into_iter()
-        .map(|api| api.typename())
+        .map(|api| api.name().clone())
+        .collect();
+    let valid_types: HashSet<_> = valid_items
+        .into_iter()
+        .map(|api| api.name())
+        .cloned()
         .collect();
-    let valid_types: HashSet<_> = valid_items.into_iter().map(|api| api.typename()).collect();
     let mut iterate_again = true;
     while iterate_again {
         iterate_again = false;
         apis = apis
             .into_iter()
             .map(|api| {
-                if api.deps.iter().any(|dep| ignored_items.contains(dep)) {
+                if api.deps().any(|dep| ignored_items.contains(dep)) {
                     iterate_again = true;
-                    ignored_items.insert(api.typename());
+                    ignored_items.insert(api.name().clone());
                     create_ignore_item(api, ConvertError::IgnoredDependent)
                 } else if !api
-                    .deps
-                    .iter()
+                    .deps()
                     .all(|dep| valid_types.contains(dep) || known_types().is_known_type(dep))
                 {
                     iterate_again = true;
-                    ignored_items.insert(api.typename());
+                    ignored_items.insert(api.name().clone());
                     create_ignore_item(api, ConvertError::UnknownDependentType)
                 } else {
                     api
@@ -67,27 +71,26 @@ pub(crate) fn filter_apis_by_ignored_dependents(
 }
 
 fn create_ignore_item(api: Api<FnAnalysis>, err: ConvertError) -> Api<FnAnalysis> {
-    let id = api.typename().get_final_ident();
-    Api {
-        name: api.typename(),
-        original_name: api.original_name,
-        deps: HashSet::new(),
-        detail: ApiDetail::IgnoredItem {
-            err,
-            ctx: match api.detail {
-                ApiDetail::Function {
-                    analysis:
-                        FnAnalysisBody {
-                            kind: FnKind::Method(self_ty, _),
-                            ..
-                        },
-                    ..
-                } => ErrorContext::Method {
-                    self_ty: self_ty.get_final_ident(),
-                    method: id,
-                },
-                _ => ErrorContext::Item(id),
+    let id = api.name().get_final_ident();
+    Api::IgnoredItem {
+        name: ApiName {
+            name: api.name().clone(),
+            cpp_name: api.cpp_name().clone(),
+        },
+        err,
+        ctx: match api {
+            Api::Function {
+                analysis:
+                    FnAnalysisBody {
+                        kind: FnKind::Method(self_ty, _),
+                        ..
+                    },
+                ..
+            } => ErrorContext::Method {
+                self_ty: self_ty.get_final_ident(),
+                method: id,
             },
+            _ => ErrorContext::Item(id),
         },
     }
 }
