@@ -18,37 +18,31 @@ use crate::conversion::{
 };
 use crate::known_types::type_lacks_copy_constructor;
 
-use super::type_to_cpp::{type_to_cpp, OriginalNameMap};
+use super::type_to_cpp::{type_to_cpp, CppNameMap};
 
 impl TypeConversionPolicy {
     pub(super) fn unconverted_type(
         &self,
-        original_name_map: &OriginalNameMap,
+        cpp_name_map: &CppNameMap,
     ) -> Result<String, ConvertError> {
         match self.cpp_conversion {
-            CppConversionType::FromUniquePtrToValue => self.wrapped_type(original_name_map),
-            _ => self.unwrapped_type_as_string(original_name_map),
+            CppConversionType::FromUniquePtrToValue => self.wrapped_type(cpp_name_map),
+            _ => self.unwrapped_type_as_string(cpp_name_map),
         }
     }
 
-    pub(super) fn converted_type(
-        &self,
-        original_name_map: &OriginalNameMap,
-    ) -> Result<String, ConvertError> {
+    pub(super) fn converted_type(&self, cpp_name_map: &CppNameMap) -> Result<String, ConvertError> {
         match self.cpp_conversion {
-            CppConversionType::FromValueToUniquePtr => self.wrapped_type(original_name_map),
-            _ => self.unwrapped_type_as_string(original_name_map),
+            CppConversionType::FromValueToUniquePtr => self.wrapped_type(cpp_name_map),
+            _ => self.unwrapped_type_as_string(cpp_name_map),
         }
     }
 
-    fn unwrapped_type_as_string(
-        &self,
-        original_name_map: &OriginalNameMap,
-    ) -> Result<String, ConvertError> {
-        type_to_cpp(&self.unwrapped_type, original_name_map)
+    fn unwrapped_type_as_string(&self, cpp_name_map: &CppNameMap) -> Result<String, ConvertError> {
+        type_to_cpp(&self.unwrapped_type, cpp_name_map)
     }
 
-    fn wrapped_type(&self, original_name_map: &OriginalNameMap) -> Result<String, ConvertError> {
+    fn wrapped_type(&self, original_name_map: &CppNameMap) -> Result<String, ConvertError> {
         Ok(format!(
             "std::unique_ptr<{}>",
             self.unwrapped_type_as_string(original_name_map)?
@@ -58,7 +52,7 @@ impl TypeConversionPolicy {
     pub(super) fn cpp_conversion(
         &self,
         var_name: &str,
-        original_name_map: &OriginalNameMap,
+        cpp_name_map: &CppNameMap,
     ) -> Result<String, ConvertError> {
         Ok(match self.cpp_conversion {
             CppConversionType::None => {
@@ -71,7 +65,7 @@ impl TypeConversionPolicy {
             CppConversionType::FromUniquePtrToValue => format!("std::move(*{})", var_name),
             CppConversionType::FromValueToUniquePtr => format!(
                 "std::make_unique<{}>({})",
-                self.unconverted_type(original_name_map)?,
+                self.unconverted_type(cpp_name_map)?,
                 var_name
             ),
         })
