@@ -22,14 +22,18 @@ mod unqualify;
 
 use std::collections::HashMap;
 
+use autocxx_parser::IncludeCppConfig;
 // The following should not need to be exposed outside
 // codegen_rs but currently Rust codegen happens everywhere... TODO
 pub(crate) use non_pod_struct::make_non_pod;
 
 use proc_macro2::TokenStream;
-use syn::{parse_quote, Attribute, ForeignItem, Ident, Item, ItemForeignMod, ItemMod};
+use syn::{parse_quote, ForeignItem, Ident, Item, ItemForeignMod, ItemMod};
 
-use crate::types::{make_ident, Namespace, QualifiedName};
+use crate::{
+    known_types::known_types,
+    types::{make_ident, Namespace, QualifiedName},
+};
 use impl_item_creator::create_impl_items;
 
 use self::{
@@ -39,11 +43,11 @@ use self::{
 };
 
 use super::codegen_cpp::type_to_cpp::{
-    namespaced_name_using_original_name_map, original_name_map_from_apis, OriginalNameMap,
+    namespaced_name_using_original_name_map, original_name_map_from_apis, CppNameMap,
 };
 use super::{
     analysis::fun::FnAnalysis,
-    api::{Api, ApiAnalysis, ApiDetail, ImplBlockDetails, TypeKind, TypedefKind},
+    api::{AnalysisPhase, Api, ImplBlockDetails, TypeKind, TypedefKind},
 };
 use super::{convert_error::ErrorContext, ConvertError};
 use quote::quote;
@@ -66,7 +70,8 @@ enum Use {
     Custom(Box<Item>),
 }
 
-fn get_string_items() -> Vec<Item> {
+fn get_string_items(config: &IncludeCppConfig) -> Vec<Item> {
+    let makestring_name = make_ident(config.get_makestring_name());
     [
         Item::Trait(parse_quote! {
             pub trait ToCppString {
@@ -79,21 +84,21 @@ fn get_string_items() -> Vec<Item> {
         Item::Impl(parse_quote! {
             impl ToCppString for &str {
                 fn into_cpp(self) -> cxx::UniquePtr<cxx::CxxString> {
-                    cxxbridge::make_string(self)
+                    cxxbridge::#makestring_name(self)
                 }
             }
         }),
         Item::Impl(parse_quote! {
             impl ToCppString for String {
                 fn into_cpp(self) -> cxx::UniquePtr<cxx::CxxString> {
-                    cxxbridge::make_string(&self)
+                    cxxbridge::#makestring_name(&self)
                 }
             }
         }),
         Item::Impl(parse_quote! {
             impl ToCppString for &String {
                 fn into_cpp(self) -> cxx::UniquePtr<cxx::CxxString> {
-                    cxxbridge::make_string(self)
+                    cxxbridge::#makestring_name(self)
                 }
             }
         }),
@@ -118,7 +123,8 @@ fn remove_nones<T>(input: Vec<Option<T>>) -> Vec<T> {
 pub(crate) struct RsCodeGenerator<'a> {
     include_list: &'a [String],
     bindgen_mod: ItemMod,
-    original_name_map: OriginalNameMap,
+    original_name_map: CppNameMap,
+    config: &'a IncludeCppConfig,
 }
 
 impl<'a> RsCodeGenerator<'a> {
@@ -127,11 +133,13 @@ impl<'a> RsCodeGenerator<'a> {
         all_apis: Vec<Api<FnAnalysis>>,
         include_list: &'a [String],
         bindgen_mod: ItemMod,
+        config: &'a IncludeCppConfig,
     ) -> Vec<Item> {
         let c = Self {
             include_list,
             bindgen_mod,
             original_name_map: original_name_map_from_apis(&all_apis),
+            config,
         };
         c.rs_codegen(all_apis)
     }
@@ -142,14 +150,15 @@ impl<'a> RsCodeGenerator<'a> {
         // what 'use' statements we need here and there.
         let generate_utilities = all_apis
             .iter()
-            .any(|api| matches!(&api.detail, ApiDetail::StringConstructor));
+            .any(|api| matches!(&api, Api::StringConstructor { .. }));
         // Now let's generate the Rust code.
         let (rs_codegen_results_and_namespaces, additional_cpp_needs): (Vec<_>, Vec<_>) = all_apis
             .into_iter()
             .map(|api| {
                 let more_cpp_needed = api.additional_cpp().is_some();
-                let gen = self.generate_rs_for_api(&api.name, api.detail);
-                ((api.name, gen), more_cpp_needed)
+                let name = api.name().clone();
+                let gen = self.generate_rs_for_api(api);
+                ((name, gen), more_cpp_needed)
             })
             .unzip();
         // First, the hierarchy of mods containing lots of 'use' statements
@@ -222,7 +231,10 @@ impl<'a> RsCodeGenerator<'a> {
 
     fn build_include_foreign_items(&self, has_additional_cpp_needs: bool) -> Vec<ForeignItem> {
         let extra_inclusion = if has_additional_cpp_needs {
-            Some("autocxxgen.h".to_string())
+            Some(format!(
+                "autocxxgen_{}.h",
+                self.config.get_mod_name().to_string()
+            ))
         } else {
             None
         };
@@ -313,54 +325,6 @@ impl<'a> RsCodeGenerator<'a> {
         }));
     }
 
-    fn remove_bindgen_attrs_from_item(mut item: Item) -> Item {
-        fn is_bindgen_attr(attr: &Attribute) -> bool {
-            let segments = &attr.path.segments;
-            segments.len() == 1
-                && segments
-                    .first()
-                    .unwrap()
-                    .ident
-                    .to_string()
-                    .starts_with("bindgen_")
-        }
-
-        fn remove_bindgen_attrs(attrs: &mut Vec<Attribute>) {
-            *attrs = attrs
-                .drain(..)
-                .filter(|attr| !is_bindgen_attr(&attr))
-                .collect();
-        }
-
-        match &mut item {
-            Item::Const(item_const) => remove_bindgen_attrs(&mut item_const.attrs),
-            Item::Enum(item_enum) => remove_bindgen_attrs(&mut item_enum.attrs),
-            Item::ExternCrate(item_extern_crate) => {
-                remove_bindgen_attrs(&mut item_extern_crate.attrs)
-            }
-            Item::Fn(item_fn) => remove_bindgen_attrs(&mut item_fn.attrs),
-            Item::ForeignMod(item_foreign_mod) => remove_bindgen_attrs(&mut item_foreign_mod.attrs),
-            Item::Impl(item_impl) => remove_bindgen_attrs(&mut item_impl.attrs),
-            Item::Macro(item_macro) => remove_bindgen_attrs(&mut item_macro.attrs),
-            Item::Macro2(item_macro2) => remove_bindgen_attrs(&mut item_macro2.attrs),
-            Item::Mod(item_mod) => remove_bindgen_attrs(&mut item_mod.attrs),
-            Item::Static(item_static) => remove_bindgen_attrs(&mut item_static.attrs),
-            Item::Struct(item_struct) => remove_bindgen_attrs(&mut item_struct.attrs),
-            Item::Trait(item_trait) => remove_bindgen_attrs(&mut item_trait.attrs),
-            Item::TraitAlias(item_trait_alias) => remove_bindgen_attrs(&mut item_trait_alias.attrs),
-            Item::Type(item_type) => remove_bindgen_attrs(&mut item_type.attrs),
-            Item::Union(item_union) => remove_bindgen_attrs(&mut item_union.attrs),
-            Item::Use(item_use) => remove_bindgen_attrs(&mut item_use.attrs),
-            Item::Verbatim(_) => {}
-            #[cfg(test)]
-            Item::__TestExhaustive(_) => unimplemented!(),
-            #[cfg(not(test))]
-            // Do nothing -- assumne there are no Attributes in the item.
-            _ => (),
-        }
-        item
-    }
-
     fn append_child_bindgen_namespace(
         &mut self,
         ns_entries: &NamespaceEntries<(QualifiedName, RsCodegenResult)>,
@@ -370,15 +334,7 @@ impl<'a> RsCodeGenerator<'a> {
     ) {
         let mut impl_entries_by_type: HashMap<_, Vec<_>> = HashMap::new();
         for item in ns_entries.entries() {
-            // Remove `bindgen_` attributes. They don't have a corresponding macro defined anywhere,
-            // so they will cause compilation errors if we leave them in.
-            output_items.extend(
-                item.1
-                    .bindgen_mod_item
-                    .iter()
-                    .cloned()
-                    .map(|item| Self::remove_bindgen_attrs_from_item(item)),
-            );
+            output_items.extend(item.1.bindgen_mod_item.iter().cloned());
             if let Some(impl_entry) = &item.1.impl_entry {
                 impl_entries_by_type
                     .entry(impl_entry.ty.clone())
@@ -434,43 +390,44 @@ impl<'a> RsCodeGenerator<'a> {
         output_items
     }
 
-    fn generate_rs_for_api(
-        &self,
-        name: &QualifiedName,
-        api_detail: ApiDetail<FnAnalysis>,
-    ) -> RsCodegenResult {
+    fn generate_rs_for_api(&self, api: Api<FnAnalysis>) -> RsCodegenResult {
+        let name = api.name().clone();
         let id = name.get_final_ident();
-        match api_detail {
-            ApiDetail::StringConstructor => RsCodegenResult {
-                extern_c_mod_item: Some(ForeignItem::Fn(parse_quote!(
-                    fn make_string(str_: &str) -> UniquePtr<CxxString>;
-                ))),
-                bridge_items: Vec::new(),
-                global_items: get_string_items(),
-                bindgen_mod_item: None,
-                impl_entry: None,
-                materialization: Use::Unused,
-            },
-            ApiDetail::ConcreteType { .. } => RsCodegenResult {
+        let cpp_call_name = api.effective_cpp_name().to_string();
+        match api {
+            Api::StringConstructor { .. } => {
+                let make_string_name = make_ident(self.config.get_makestring_name());
+                RsCodegenResult {
+                    extern_c_mod_item: Some(ForeignItem::Fn(parse_quote!(
+                        fn #make_string_name(str_: &str) -> UniquePtr<CxxString>;
+                    ))),
+                    bridge_items: Vec::new(),
+                    global_items: get_string_items(self.config),
+                    bindgen_mod_item: None,
+                    impl_entry: None,
+                    materialization: Use::Unused,
+                }
+            }
+            Api::ConcreteType { .. } => RsCodegenResult {
                 global_items: self.generate_extern_type_impl(TypeKind::NonPod, &name),
-                bridge_items: create_impl_items(&id),
-                extern_c_mod_item: Some(ForeignItem::Verbatim(self.generate_cxxbridge_type(name))),
+                bridge_items: create_impl_items(&id, self.config),
+                extern_c_mod_item: Some(ForeignItem::Verbatim(self.generate_cxxbridge_type(&name))),
                 bindgen_mod_item: Some(Item::Struct(new_non_pod_struct(id.clone()))),
                 impl_entry: None,
                 materialization: Use::Unused,
             },
-            ApiDetail::ForwardDeclaration => RsCodegenResult {
-                extern_c_mod_item: Some(ForeignItem::Verbatim(self.generate_cxxbridge_type(name))),
+            Api::ForwardDeclaration { .. } => RsCodegenResult {
+                extern_c_mod_item: Some(ForeignItem::Verbatim(self.generate_cxxbridge_type(&name))),
                 bridge_items: Vec::new(),
                 global_items: self.generate_extern_type_impl(TypeKind::NonPod, &name),
                 bindgen_mod_item: Some(Item::Struct(new_non_pod_struct(id))),
                 impl_entry: None,
                 materialization: Use::UsedFromCxxBridge,
             },
-            ApiDetail::Function { fun, analysis } => {
-                gen_function(name.get_namespace(), *fun, analysis)
+            Api::Function { fun, analysis, .. } => {
+                gen_function(name.get_namespace(), *fun, analysis, cpp_call_name)
             }
-            ApiDetail::Const { const_item } => RsCodegenResult {
+            Api::Const { const_item, .. } => RsCodegenResult {
                 global_items: Vec::new(),
                 impl_entry: None,
                 bridge_items: Vec::new(),
@@ -478,33 +435,24 @@ impl<'a> RsCodeGenerator<'a> {
                 bindgen_mod_item: Some(Item::Const(const_item)),
                 materialization: Use::UsedFromBindgen,
             },
-            ApiDetail::Typedef { payload } => RsCodegenResult {
+            Api::Typedef { analysis, .. } => RsCodegenResult {
                 extern_c_mod_item: None,
                 bridge_items: Vec::new(),
                 global_items: Vec::new(),
-                bindgen_mod_item: Some(match payload {
+                bindgen_mod_item: Some(match analysis.kind {
                     TypedefKind::Type(type_item) => Item::Type(type_item),
                     TypedefKind::Use(use_item) => Item::Use(use_item),
                 }),
                 impl_entry: None,
                 materialization: Use::UsedFromBindgen,
             },
-            ApiDetail::Type {
-                bindgen_mod_item,
-                analysis,
-            } => RsCodegenResult {
-                global_items: self.generate_extern_type_impl(analysis, &name),
-                impl_entry: None,
-                bridge_items: if analysis.can_be_instantiated() {
-                    create_impl_items(&id)
-                } else {
-                    Vec::new()
-                },
-                extern_c_mod_item: Some(ForeignItem::Verbatim(self.generate_cxxbridge_type(name))),
-                bindgen_mod_item,
-                materialization: Use::UsedFromCxxBridge,
-            },
-            ApiDetail::CType { .. } => RsCodegenResult {
+            Api::Struct { item, analysis, .. } => {
+                self.generate_type(&name, id, item, analysis.kind, Item::Struct)
+            }
+            Api::Enum { item, .. } => {
+                self.generate_type(&name, id, item, TypeKind::Pod, Item::Enum)
+            }
+            Api::CType { .. } => RsCodegenResult {
                 global_items: Vec::new(),
                 impl_entry: None,
                 bridge_items: Vec::new(),
@@ -514,7 +462,32 @@ impl<'a> RsCodeGenerator<'a> {
                 bindgen_mod_item: None,
                 materialization: Use::Unused,
             },
-            ApiDetail::IgnoredItem { err, ctx } => Self::generate_error_entry(err, ctx),
+            Api::IgnoredItem { err, ctx, .. } => Self::generate_error_entry(err, ctx),
+        }
+    }
+
+    fn generate_type<T, F>(
+        &self,
+        name: &QualifiedName,
+        id: Ident,
+        item: T,
+        analysis: TypeKind,
+        item_type: F,
+    ) -> RsCodegenResult
+    where
+        F: FnOnce(T) -> Item,
+    {
+        RsCodegenResult {
+            global_items: self.generate_extern_type_impl(analysis, &name),
+            impl_entry: None,
+            bridge_items: if analysis.can_be_instantiated() {
+                create_impl_items(&id, self.config)
+            } else {
+                Vec::new()
+            },
+            extern_c_mod_item: Some(ForeignItem::Verbatim(self.generate_cxxbridge_type(name))),
+            bindgen_mod_item: Some(item_type(item)),
+            materialization: Use::UsedFromCxxBridge,
         }
     }
 
@@ -524,24 +497,50 @@ impl<'a> RsCodeGenerator<'a> {
     fn generate_error_entry(err: ConvertError, ctx: ErrorContext) -> RsCodegenResult {
         let err = format!("autocxx bindings couldn't be generated: {}", err);
         let (impl_entry, materialization) = match ctx {
-            ErrorContext::Item(id) => (
-                None,
-                Use::Custom(Box::new(parse_quote! {
-                    #[doc = #err]
-                    pub struct #id;
-                })),
-            ),
-            ErrorContext::Method { self_ty, method } => (
-                Some(Box::new(ImplBlockDetails {
-                    item: parse_quote! {
+            ErrorContext::Item(id) => {
+                let id = Self::sanitize_error_ident(&id).unwrap_or(id);
+                (
+                    None,
+                    Use::Custom(Box::new(parse_quote! {
                         #[doc = #err]
-                        fn #method(_uhoh: autocxx::BindingGenerationFailure) {
-                        }
-                    },
-                    ty: self_ty,
-                })),
-                Use::Unused,
-            ),
+                        pub struct #id;
+                    })),
+                )
+            }
+            ErrorContext::Method { self_ty, method }
+                if Self::sanitize_error_ident(&self_ty).is_none() =>
+            {
+                // This could go wrong if a separate error has caused
+                // us to drop an entire type as well as one of its individual items.
+                // Then we'll be applying an impl to a thing which doesn't exist. TODO.
+                let method = Self::sanitize_error_ident(&method).unwrap_or(method);
+                (
+                    Some(Box::new(ImplBlockDetails {
+                        item: parse_quote! {
+                            #[doc = #err]
+                            fn #method(_uhoh: autocxx::BindingGenerationFailure) {
+                            }
+                        },
+                        ty: self_ty,
+                    })),
+                    Use::Unused,
+                )
+            }
+            ErrorContext::Method { self_ty, method } => {
+                // If the type can't be represented (e.g. u8) this would get fiddly.
+                let id = make_ident(format!(
+                    "{}_method_{}",
+                    self_ty.to_string(),
+                    method.to_string()
+                ));
+                (
+                    None,
+                    Use::Custom(Box::new(parse_quote! {
+                        #[doc = #err]
+                        pub struct #id;
+                    })),
+                )
+            }
         };
         RsCodegenResult {
             global_items: Vec::new(),
@@ -553,6 +552,17 @@ impl<'a> RsCodeGenerator<'a> {
         }
     }
 
+    /// Because errors may be generated for invalid types or identifiers,
+    /// we may need to scrub the name
+    fn sanitize_error_ident(id: &Ident) -> Option<Ident> {
+        let qn = QualifiedName::new(&Namespace::new(), id.clone());
+        if known_types().conflicts_with_built_in_type(&qn) {
+            Some(make_ident(format!("{}_autocxx_error", qn.get_final_item())))
+        } else {
+            None
+        }
+    }
+
     fn generate_cxx_use_stmt(name: &QualifiedName, alias: Option<&Ident>) -> Item {
         let segs = Self::find_output_mod_root(name.get_namespace())
             .chain(std::iter::once(make_ident("cxxbridge")))
@@ -594,12 +604,12 @@ impl<'a> RsCodeGenerator<'a> {
     fn generate_cxxbridge_type(&self, name: &QualifiedName) -> TokenStream {
         let ns = name.get_namespace();
         let id = name.get_final_ident();
-        let mut ns_components: Vec<String> = ns.iter().cloned().collect();
+        let mut ns_components: Vec<_> = ns.iter().cloned().collect();
         let mut cxx_name = None;
-        if let Some(original_name) = self.original_name_map.get(name) {
-            let mut name_components: Vec<_> = original_name.split("::").collect();
-            cxx_name = Some(name_components.pop().unwrap());
-            ns_components.extend(name_components.iter().map(|s| s.to_string()));
+        if let Some(cpp_name) = self.original_name_map.get(name) {
+            let cpp_name = QualifiedName::new_from_cpp_name(cpp_name);
+            cxx_name = Some(cpp_name.get_final_item().to_string());
+            ns_components.extend(cpp_name.ns_segment_iter().cloned());
         };
 
         let mut for_extern_c_ts = if !ns_components.is_empty() {
@@ -642,9 +652,9 @@ impl HasNs for (QualifiedName, RsCodegenResult) {
     }
 }
 
-impl<T: ApiAnalysis> HasNs for Api<T> {
+impl<T: AnalysisPhase> HasNs for Api<T> {
     fn get_namespace(&self) -> &Namespace {
-        &self.name.get_namespace()
+        &self.name().get_namespace()
     }
 }
 
