@@ -40,7 +40,6 @@ pub enum ConvertError {
     TemplatedTypeContainingNonPathArg(QualifiedName),
     InvalidPointee,
     DidNotGenerateAnything(String),
-    UnacceptableSpecialNamespaceType(QualifiedName),
     TypeContainingForwardDeclaration(QualifiedName),
     Blocked(QualifiedName),
     UnusedTemplateParam,
@@ -48,6 +47,9 @@ pub enum ConvertError {
     UnknownDependentType,
     IgnoredDependent,
     MoveConstructorUnsupported,
+    ReservedName,
+    DuplicateCxxBridgeName,
+    UnsupportedReceiver,
 }
 
 fn format_maybe_identifier(id: &Option<Ident>) -> String {
@@ -80,7 +82,6 @@ impl Display for ConvertError {
             ConvertError::TemplatedTypeContainingNonPathArg(tn) => write!(f, "Type {} was parameterized over something complex which we don't yet support", tn)?,
             ConvertError::InvalidPointee => write!(f, "Pointer pointed to something unsupported")?,
             ConvertError::DidNotGenerateAnything(directive) => write!(f, "The 'generate' or 'generate_pod' directive for '{}' did not result in any code being generated. Perhaps this was mis-spelled or you didn't qualify the name with any namespaces? Otherwise please report a bug.", directive)?,
-            ConvertError::UnacceptableSpecialNamespaceType(tn) => write!(f, "The type '{}' would require special support from cxx/autocxx, but is not yet supported", tn.to_cpp_name())?,
             ConvertError::TypeContainingForwardDeclaration(tn) => write!(f, "Found an attempt at using a forward declaration ({}) inside a templated cxx type such as UniquePtr or CxxVector", tn.to_cpp_name())?,
             ConvertError::Blocked(tn) => write!(f, "Found an attempt at using a type marked as blocked! ({})", tn.to_cpp_name())?,
             ConvertError::UnusedTemplateParam => write!(f, "This function or method uses a type where one of the template parameters was incomprehensible to bindgen/autocxx - probably because it uses template specialization.")?,
@@ -88,11 +89,15 @@ impl Display for ConvertError {
             ConvertError::UnknownDependentType => write!(f, "This item relies on a type not known to autocxx.")?,
             ConvertError::IgnoredDependent => write!(f, "This item depends on some other type which autocxx could not generate.")?,
             ConvertError::MoveConstructorUnsupported => write!(f, "This is a move constructor, for which we currently cannot generate bindings.")?,
+            ConvertError::ReservedName => write!(f, "The item name is a reserved word in Rust.")?,
+            ConvertError::DuplicateCxxBridgeName => write!(f, "This item name is used in multiple namespaces. At present, autocxx and cxx allow only one type of a given name. This limitation will be fixed in future.")?,
+            ConvertError::UnsupportedReceiver => write!(f, "This is a method on a type which can't be used as the receiver in Rust (i.e. self/this). This is probably because some type involves template specialization.")?,
         }
         Ok(())
     }
 }
 
+#[derive(Clone)]
 pub(crate) enum ErrorContext {
     Item(Ident),
     Method { self_ty: Ident, method: Ident },
