@@ -71,7 +71,7 @@ impl TypeDetails {
             | Behavior::CxxString
             | Behavior::CxxContainerByValueSafe
             | Behavior::CxxContainerNotByValueSafe => {
-                let tn = QualifiedName::new_from_user_input(&self.rs_name);
+                let tn = QualifiedName::new_from_cpp_name(&self.rs_name);
                 let cxx_name = tn.get_final_item();
                 let (templating, payload) = match self.behavior {
                     Behavior::CxxContainerByValueSafe | Behavior::CxxContainerNotByValueSafe => {
@@ -97,14 +97,23 @@ impl TypeDetails {
     }
 
     fn to_type_path(&self) -> TypePath {
-        let segs = self.rs_name.split("::").map(make_ident);
-        parse_quote! {
-            #(#segs)::*
+        let mut segs = self.rs_name.split("::").peekable();
+        if segs.peek().map(|seg| seg.is_empty()).unwrap_or_default() {
+            segs.next();
+            let segs = segs.into_iter().map(make_ident);
+            parse_quote! {
+                ::#(#segs)::*
+            }
+        } else {
+            let segs = segs.into_iter().map(make_ident);
+            parse_quote! {
+                #(#segs)::*
+            }
         }
     }
 
     fn to_typename(&self) -> QualifiedName {
-        QualifiedName::new_from_user_input(&self.rs_name)
+        QualifiedName::new_from_cpp_name(&self.rs_name)
     }
 }
 
@@ -183,26 +192,8 @@ impl TypeDatabase {
     /// We strip off and ignore
     /// any PathArguments within this TypePath - callers should
     /// put them back again if needs be.
-    pub(crate) fn consider_substitution(
-        &self,
-        tn: &QualifiedName,
-    ) -> Result<Option<TypePath>, ConvertError> {
-        match self.get(&tn) {
-            None => {
-                // Only allow types from std:: and rust:: which we specifically understand,
-                // because we've told bindgen to block all the rest.
-                if tn
-                    .ns_segment_iter()
-                    .next()
-                    .filter(|seg| BINDGEN_BLOCKLIST_NAMESPACES.iter().any(|ns| ns == seg))
-                    .is_some()
-                {
-                    return Err(ConvertError::UnacceptableSpecialNamespaceType(tn.clone()));
-                }
-                Ok(None)
-            }
-            Some(td) => Ok(Some(td.to_type_path())),
-        }
+    pub(crate) fn consider_substitution(&self, tn: &QualifiedName) -> Option<TypePath> {
+        self.get(&tn).map(|td| td.to_type_path())
     }
 
     pub(crate) fn special_cpp_name(&self, rs: &QualifiedName) -> Option<String> {
@@ -217,6 +208,14 @@ impl TypeDatabase {
         self.get(ty).map(|td| td.to_type_path())
     }
 
+    /// Get the list of types to give to bindgen to ask it _not_ to
+    /// generate code for.
+    pub(crate) fn get_initial_blocklist(&self) -> impl Iterator<Item = &str> + '_ {
+        self.by_rs_name
+            .iter()
+            .filter_map(|(_, td)| td.get_prelude_entry().map(|_| td.cpp_name.as_str()))
+    }
+
     /// Whether this is one of the ctypes (mostly variable length integers)
     /// which we need to wrap.
     pub(crate) fn is_ctype(&self, ty: &QualifiedName) -> bool {
@@ -244,6 +243,15 @@ impl TypeDatabase {
             .unwrap_or(false)
     }
 
+    pub(crate) fn is_cxx_acceptable_receiver(&self, ty: &QualifiedName) -> bool {
+        self.get(ty).is_none() // at present, none of our known types can have
+                               // methods attached.
+    }
+
+    pub(crate) fn conflicts_with_built_in_type(&self, ty: &QualifiedName) -> bool {
+        self.get(ty).is_some()
+    }
+
     pub(crate) fn convertible_from_strs(&self, ty: &QualifiedName) -> bool {
         self.get(ty)
             .map(|x| matches!(x.behavior, Behavior::CxxString))
@@ -254,12 +262,12 @@ impl TypeDatabase {
         let rs_name = td.to_typename();
         if let Some(extra_non_canonical_name) = &td.extra_non_canonical_name {
             self.canonical_names.insert(
-                QualifiedName::new_from_user_input(extra_non_canonical_name),
+                QualifiedName::new_from_cpp_name(extra_non_canonical_name),
                 rs_name.clone(),
             );
         }
         self.canonical_names.insert(
-            QualifiedName::new_from_user_input(&td.cpp_name),
+            QualifiedName::new_from_cpp_name(&td.cpp_name),
             rs_name.clone(),
         );
         self.by_rs_name.insert(rs_name, td);
@@ -286,6 +294,12 @@ fn create_type_database() -> TypeDatabase {
         Behavior::CxxContainerByValueSafe,
         None,
     ));
+    db.insert(TypeDetails::new(
+        "cxx::WeakPtr",
+        "std::weak_ptr",
+        Behavior::CxxContainerByValueSafe,
+        None,
+    ));
     db.insert(TypeDetails::new(
         "cxx::CxxString",
         "std::string",
@@ -366,7 +380,7 @@ fn create_type_database() -> TypeDatabase {
     db.insert(TypeDetails::new("f32", "float", Behavior::CByValue, None));
     db.insert(TypeDetails::new("f64", "double", Behavior::CByValue, None));
     db.insert(TypeDetails::new(
-        "std::os::raw::c_char",
+        "::std::os::raw::c_char",
         "char",
         Behavior::CByValue,
         None,
@@ -380,30 +394,6 @@ fn create_type_database() -> TypeDatabase {
     db
 }
 
-/// Namespaces which we pass to bindgen's blocklist.
-/// This is because we have special knowledge and handling of
-/// types in those namespaces.
-const BINDGEN_BLOCKLIST_NAMESPACES: &[&str] = &["std", "rust"];
-
-/// Extra things to include in the bindgen blocklist.
-/// This is worked out basically using trial and error.
-/// Excluding std* and rust* is obvious, but the other items...
-/// in theory bindgen ought to be smart enough to work out that
-/// they're not used and therefore not generate code for them.
-/// But it doesm unless we blocklist them. This is obviously
-/// a bit sensitive to the particular STL in use so one day
-/// it would be good to dig into bindgen's behavior here - TODO.
-const BINDGEN_BLOCKLIST: &[&str] = &["__gnu.*", ".*mbstate_t.*"];
-
-/// Get the list of types to give to bindgen to ask it _not_ to
-/// generate code for.
-pub(crate) fn get_initial_blocklist() -> impl Iterator<Item = String> {
-    BINDGEN_BLOCKLIST_NAMESPACES
-        .iter()
-        .map(|ns| format!("{}::.*", ns))
-        .chain(BINDGEN_BLOCKLIST.iter().map(|s| s.to_string()))
-}
-
 /// If a given type lacks a copy constructor, we should always use
 /// std::move in wrapper functions.
 pub(crate) fn type_lacks_copy_constructor(ty: &Type) -> bool {
